# frozen_string_literal: true

module ServerComponent
  class FetcherConfigurator
    include ServerComponentHelper

    def initialize(component_class, actions)
      @component_class = component_class
      @actions = actions
    end

    def before
      jsrb = Jsrb::Base.new
      set_state(jsrb) do |s|
        yield(s, jsrb.expr.context.data)
      end
      set_value(:start_function, jsrb.generate_code)
    end

    def accept_file!
      set_value :upload, true
    end

    def set_value(key, value)
      @actions.each do |action|
        @component_class.action_descriptors[action][key] = value
      end
    end
  end

  module Component
    extend ActiveSupport::Concern

    included do
      class_attribute :initial_state, :action_on_mount

      helper ServerComponentHelper
      include ServerComponentHelper

      private

      def set_state(&block) # rubocop:disable Naming/AccessorMethodName
        jsrb = Jsrb::Base.new
        super(jsrb, &block)
        render jsrb: jsrb.generate_code
      end
    end

    class_methods do
      def to_json
        {
          action_on_mount: action_on_mount,
          initial_state: initial_state,
          actions: action_descriptors
        }
      end

      def action_descriptors
        if instance_variable_defined?('@action_descriptors')
          instance_variable_get '@action_descriptors'
        else
          h = {}
          instance_variable_set '@action_descriptors', h
          h
        end
      end

      def action(*action_names, &block)
        options = action_names.extract_options!
        raise ArgumentError, 'specify at least one action!' if action_names.empty?

        action_names.each do |action_name|
          action_descriptors[action_name.to_sym] = {}
        end
        fetcher(*action_names, &block) if block
        if options[:on_mount]
          raise ArgumentError, 'on_mount option can accept only one action' if action_names.size > 1

          on_mount action_names.first
        end
      end

      def on_mount(action)
        self.action_on_mount = action
      end

      def fetcher(*actions)
        configurator = FetcherConfigurator.new(self, actions.map(&:to_sym))
        yield configurator
      end

      def before(*actions, &block)
        fetcher(*actions) do |c|
          c.before(&block)
        end
      end
    end
  end
end
